@@ -9,9 +9,11 @@ pub mod list;
 pub mod get_old;
 pub mod get_id;
 pub mod get_refs;
-pub mod json_dir_to_rust;
 pub mod construct_root;
 pub mod validation;
+pub mod set_empty_mils;
+pub mod roots;
+
 use dochy_json5::JVal;
 use names::Names;
 use crate::error::CoreResult;
