@@ -1,8 +1,8 @@
 use crate::imp::rust_to_json::list::default_to_json::default_to_json;
 use crate::imp::structs::my_json::Value;
-use crate::imp::structs::mil_def_obj::MilDefObj;
+use crate::imp::structs::mut_list_def::MutListDef;
 
-pub(crate) fn inner_mut_def_to_json(d : &MilDefObj) -> Value{
+pub(crate) fn inner_mut_def_to_json(d : &MutListDef) -> Value{
     let mut result : Vec<Value> = Vec::new();
 
     result.push(Value::String("MilDef".to_string()));
@@ -10,7 +10,7 @@ pub(crate) fn inner_mut_def_to_json(d : &MilDefObj) -> Value{
     //     result.push(string_set_to_json("Compatible", &btree_set(d.compatible())));
     // }
 
-    result.push(Value::Array(vec![default_to_json(d.list_def())]));
+    result.push(Value::Array(vec![default_to_json(d.default())]));
 
 
     return Value::Array(result);
