@@ -16,15 +16,15 @@ impl RustArray{
     }
 
     pub(crate) fn from_int_array(qv : &Qv<RustIntArray>) -> RustArray{
-        RustArray::new(qv.map(|a| a.to_params()))
+        RustArray::new(qv.as_ref().map(|a| a.to_params()))
     }
 
     pub(crate) fn from_float_array(qv : &Qv<RustFloatArray>) -> RustArray{
-        RustArray::new(qv.map(|a| a.to_params()))
+        RustArray::new(qv.as_ref().map(|a| a.to_params()))
     }
 
     pub(crate) fn from_binary(qv : &Qv<RustBinary>) -> RustArray{
-        RustArray::new(qv.map(|a| a.to_params()))
+        RustArray::new(qv.as_ref().map(|a| a.to_params()))
     }
 
     pub(crate) fn qv(&self) -> &Qv<Vec<RustParam>>{ self.array.as_ref() }
@@ -52,7 +52,7 @@ impl RustArray{
     }
 }
 
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, PartialEq)]
 pub struct RustFloatArray{
     b : Box<Vec<f64>>,
 }
@@ -77,13 +77,13 @@ impl IdentityEqual for RustFloatArray{
     }
 }
 
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, PartialEq)]
 pub struct RustIntArray{
-    b : Box<Vec<i64>>,
+    b : Arc<Vec<i64>>,
 }
 
 impl RustIntArray{
-    pub fn new(b : Vec<i64>) -> RustIntArray{ RustIntArray{ b : Box::new(b) }}
+    pub fn new(b : Vec<i64>) -> RustIntArray{ RustIntArray{ b : Arc::new(b) }}
     //pub(crate) fn as_ref(&self) -> &Vec<i64>{ self.b.as_ref() }
     pub(crate) fn to_params(&self) -> Vec<RustParam>{
         self.vec().iter().map(|a| RustParam::Int(Qv::Val(*a))).collect()
@@ -93,16 +93,16 @@ impl RustIntArray{
         Some(RustIntArray::new(op?))
     }
     pub fn vec(&self) -> &Vec<i64>{ self.b.as_ref() }
-    pub fn vec_mut(&mut self) -> &mut Vec<i64>{ self.b.as_mut() }
+    pub fn vec_mut(&mut self) -> &mut Vec<i64>{ Arc::make_mut(&mut self.b) }
 }
 
 impl IdentityEqual for RustIntArray{
     fn identity_eq(&self, other: &Self) -> bool {
-        self.b == other.b
+        Arc::ptr_eq(&self.b, &other.b)
     }
 }
 
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, PartialEq)]
 pub struct RustBinary{
     b : Arc<Vec<u8>>,
 }
